/*++

Copyright (c) 2009
Lysann Kessler, Stephanie Platz, Thomas Klingbeil, Philipp Tessenow, Frank Schlegel;
Michael Haupt, Michael Perscheid, Robert Hirschfeld;
Software Architecture Group, Hasso-Plattner-Institut, University of Potsdam, Germany

This software is published under MIT license.
It uses phidget library by Phidgets Inc. (http://phidgets.com).
See ../../../../license.pdf for more information.

--*/

#include "PhidgetPlugin.h"

//////////////////////////////////////////////////////////////////////////
// global variables

// Last phidget error code. See the phidget library for more information
// about phidget error codes.
int g_iLastError;
// Global Phidgets list. This is actually the pointer to the first element,
// or NULL if the list is empty.
Phidget* g_phidgetList;

//////////////////////////////////////////////////////////////////////////
// Plugin Functions

/*++

Routine Description:

	This functions is called upon loading of the plugin, i.e. just before
	the first primitve is called.
	Reset the last error code and the global list of phidgets.
	In debug mode also reset the debug file pointer.

Arguments:

	None.

Return Value:

	None.

--*/
void phidgetPlugin_Initialise()
{
	FILE* f;

	// start logging
	IF_DBG(SLog_enableLogging_ex(PHID_LOGFILE, "w", PHID_BASIC_LOG_LEVEL, PHID_DBG, SLOG_TRUE));
	IF_DBG(SLog(SLOG_LEVEL_INFO, "Initialising Plugin"));

	// reset global variables
	g_iLastError = 0;
	g_phidgetList = NULL;
	phidgetManager_initialise();
}

/*++

Routine Description:

	This functions is called when unloading the plugin, i.e. when
	shutting down the Squeak VM or when explicitly unloading the
	plugin in Squeak.
	Close the hardware connections to all currently still opened
	Phidgets.

Arguments:

	None.

Return Value:

	None.

--*/
void phidgetPlugin_Shutdown()
{
	Phidget* pIterator, *pNext;

	IF_DBG(SLog(SLOG_LEVEL_INFO, "Sutting down Plugin..."));

	// close phidget manager
	phidgetManager_shutDown();

	// close all still opened Phidgets
	for(pIterator = g_phidgetList; pIterator != NULL; pIterator = pNext)
	{
		// save the next list item before the current item is removed from the list
		pNext = pIterator->pNextLink;
		phidget_Close(pIterator->id);
		phidgetPlugin_RemovePhidget(pIterator);
	}
	g_phidgetList = NULL;
	IF_DBG(SLog(SLOG_LEVEL_INFO, "All Phidgets closed"));

	// stop logging
	IF_DBG(SLog_disableLogging());
}

PhidgetId phidgetPlugin_AssureUniqueId(PhidgetId id)
{
	PhidgetId biggestId;

	// catch -1 and check that the id is bigger than all previous IDs.
	// As all new Phidgets are only added at front of the list
	// we only need to check the first item.

	biggestId = g_phidgetList != NULL ? g_phidgetList->id : 0;

	if(id > biggestId) return id;
	else return biggestId + 1;
}

// TODO: kommentieren
PhidgetId phidgetPlugin_GetNextPhidgetId()
{
	time_t now;

	// get current time as new time stamp
	now = time(NULL);
	if(now == -1)
		IF_DBG(SLog(SLOG_LEVEL_CRITICAL, "Cannot get current time stamp in phidgetPlugin_getNextPhidgetId()"));

	return phidgetPlugin_AssureUniqueId(now);
}

/*++

Routine Description:

	This functions adds a phidget object to the global list
	of phidgets (g_phidgetList).

Arguments:

	pPhidget - Pointer to the phidget object to be added.

Return Value:

	FALSE if the given pointer is NULL.
	TRUE if the phidget was successfully added to the list.

--*/
BOOL phidgetPlugin_AddPhidget(Phidget* pPhidget)
{
	if(pPhidget == NULL)
	{
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Null pointer given in phidgetPlugin_AddPhidget for pPhidget."));
		return FALSE;
	}

	// add at front of list, because this is quite simple =)
	IF_DBG(SLog(SLOG_LEVEL_INFO, "Adding new Phidget to global list: ID=%d", pPhidget->id));
	pPhidget->pNextLink = g_phidgetList;
	g_phidgetList = pPhidget;

	return TRUE;
}

/*++

Routine Description:

	This functions removes a phidget object from the global list
	of phidgets (g_phidgetList), and free its memory.

Arguments:

	pPhidget - Pointer to the phidget object to be removed.

Return Value:

	FALSE if the given pointer is NULL, or if the phidget is
	not part of the list.
	TRUE if the phidget was successfully removed from the list.

Note:

	This functions is currently only used in phidgetPlugin_Shutdown(),
	but there it is obsolete (as the list will then soon be erased).
	Do we need this function? Do we need an "delete" function for
	phidget objects (where this method is called)?

---TODO---

--*/
BOOL phidgetPlugin_RemovePhidget(Phidget* pPhidget)
{
	Phidget* pIterator, *pNext;

	if(pPhidget == NULL)
	{
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Null pointer given in phidgetPlugin_RemovePhidget for pPhidget."));
		return FALSE;
	}

	IF_DBG(SLog(SLOG_LEVEL_INFO, "Removing Phidget from global list: ID=%d", pPhidget->id));
	
	if(g_phidgetList == pPhidget) // is it the first entry?
	{
		// remove it
		pIterator = g_phidgetList;
		g_phidgetList = pIterator->pNextLink;

		// free its memory
		free(pIterator);

		return TRUE;
	}
	for(pIterator = g_phidgetList; pIterator != NULL; pIterator = pNext)
	{
		pNext = pIterator->pNextLink;
		if(pNext == pPhidget) // is not NULL!
		{
			// remove next item
			pIterator->pNextLink = pNext->pNextLink;
			
			// free its memory
			free(pNext);
			
			return TRUE;
		}
	}

	IF_DBG(SLog(SLOG_LEVEL_WARNING, "Trying to remove Phidget from global list, but it could not be found: ID=%d", pPhidget->id));
	return FALSE;
}

/*++

Routine Description:

	This functions is used to get the corresponding phidget
	object from its ID. It searches for an object with the
	same ID in the global phidgets list (g_phidgetList).

Arguments:

	id - ID of the searched phidget object. This id is provided
		upon creation of the phidget.

Return Value:

	A pointer to the searched phidget object if it is in the list.
	NULL if no such object exists.

--*/
Phidget* phidgetPlugin_GetPhidgetFromId(PhidgetId id)
{
	Phidget* pIterator;

	for(pIterator = g_phidgetList; pIterator != NULL; pIterator = pIterator->pNextLink)
	{
		if(pIterator->id == id)
			return pIterator;
	}

	IF_DBG(SLog(SLOG_LEVEL_ERROR, "Trying to get Phidget from ID=%d, but no such Phidget exists!", id));
	return NULL;
}

/*++

Routine Description:

	This is a helper function that makes a zero-terminated string
	(so called C-String) from a character array of given length.
	Use this routine to work with functions that need a zero-
	terminated string but you cannot be sure a character array
	you have is zero-terminated.

Arguments:

	pcString - Pointer to a character array to be converted to a
		C-String.

	iLength - Size of the latter array.

Return Value:

	NULL if the given pointer is NULL or memory cannot be allocated.
	Else a pointer to a newly allocated C-String.

Note:

	Remember to call phidgetPlugin_FreeCString() when finished with
	the returned C-String to free allocated memory.

--*/
char* phidgetPlugin_ConvertToCString(char* pcString, int iLength)
{
	char* pszString;
	
	if(pcString == NULL)
	{
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Null pointer given in phidgetPlugin_ConvertToCString for pcString."));
		return NULL;
	}

	pszString = calloc(iLength + 1, sizeof(char));
	if(pszString == NULL)
	{
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Cannot allocate %i bytes in phidgetPlugin_ConvertToCString.", (iLength + 1) * sizeof(char)));
		return NULL;
	}
	memcpy(pszString, pcString, iLength);
	return pszString;
}

/*++

Routine Description:

	This is a helper function that releases memory previously
	allocated by phidgetPlugin_ConvertToCString().

Arguments:

	pszString - Pointer to a character array that was converted
		to a C-String.

Return Value:

	None.

--*/
void phidgetPlugin_FreeCString(char* pszString)
{
	if(pszString == NULL)
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Null pointer given in phidgetPlugin_FreeCString for pszString."));
	else
		free(pszString);
}
