/*++

Copyright (c) 2009
Lysann Kessler, Stephanie Platz, Thomas Klingbeil, Philipp Tessenow, Frank Schlegel;
Michael Haupt, Michael Perscheid, Robert Hirschfeld;
Software Architecture Group, Hasso-Plattner-Institut, University of Potsdam, Germany

This software is published under MIT license.
It uses phidget library by Phidgets Inc. (http://phidgets.com).
See ../../../../license.pdf for more information.

Module Name:

	PhidgetAdvancedServo.c

Abstract:

	This file contains all Phidget Advanced Servo specific methods except
	for those the PhidgetServo.c implements yet. The Advanced Servo is a
	special Servo so that it (and its component) can use the Phidget Servo
	specific methods, too. A Phidget Advnaced Servo has a component 
	advanced servo motor. Both modules - PhidgetCompAdvancedServoMotor.c
	and PhidgetAdvancedServo.c - with the modules implement all methods 
	supplied by the phidget21 API for an Advanced Servo. 

--*/

#include "PhidgetAdvancedServo.h"

//////////////////////////////////////////////////////////////////////////
// forward declarations: callback functions

int CALLBACK phidgetAdvServo_currentChangeHandler(CPhidgetAdvancedServoHandle advServoHandle, void *pUserPtr, int iIndex, double dCurrent);
int CALLBACK phidgetAdvServo_velocityChangeHandler(CPhidgetAdvancedServoHandle advServoHandle, void *pUserPtr, int iIndex, double dVelocity);

//////////////////////////////////////////////////////////////////////////
// PhidgetAdvancedServo methods

/*++

Routine Description:

	This method creates an PhidgetAdvServo struct and allocates memory.
	To identify the phidgets type the deviceClass is set here.
	Additionally do common creational settings.

Arguments:

	None.

Return Value:

	Squeak VM identifier for the PhigdetId Object.

--*/
sqInt phidgetAdvServo_Create()
{
	// allocate memory for a Advanced Servo structure
	PhidgetAdvServo* pPhidget = calloc(1, sizeof(PhidgetAdvServo));

	// initialise
	pPhidget->deviceClass = PHIDCLASS_ADVANCEDSERVO;
	phidgetAdvServo_ResetSemaphores(pPhidget);

	// get id and add to global list. return the id
	return phidget_Create((Phidget*)pPhidget);
}

//
//reset semaphores
//
void phidgetAdvServo_ResetSemaphores(PhidgetAdvServo *pPhidget)
{
	if(pPhidget != NULL)
	{
		phidget_ResetSemaphores((Phidget*)pPhidget);

		pPhidget->iSemaphoreVelocityChange = 0;
		pPhidget->iSemaphoreCurrentChange = 0;
	}
	else
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Null pointer given in phidgetAdvServo_ResetSemaphores for pPhidget."));
}


/*++

Routine Description:

	This method is called while opening a phidget. The actuall specific
	phidget object will be created and the specific callbacks will be
	registered.

Arguments:

	pPhidget - pointer of the Phidget to open the hardware connection.

Return Value:

	TRUE if the operation succeeded, FALSE otherwise.

--*/
BOOL phidgetAdvServo_Open(PhidgetAdvServo* pPhidget)
{
	if(pPhidget == NULL)
	{
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Null pointer given in phidgetAdvServo_Open for pPhidget."));
		return FALSE;
	}

	//create the Advanced Servo object
	pPhidget->handle = 0;
	CPhidgetAdvancedServo_create(&(pPhidget->handle));

	//Registers the callbacks that will run if the current draw (circuit) or a velocity changes
	//Requires the handle for the Phidget, the function that will be called, and an arbitrary pointer that will be supplied to the callback function (may be NULL).
	CPhidgetAdvancedServo_set_OnVelocityChange_Handler (pPhidget->handle, phidgetAdvServo_velocityChangeHandler,(void *)pPhidget->id);
	CPhidgetAdvancedServo_set_OnCurrentChange_Handler (pPhidget->handle, phidgetAdvServo_currentChangeHandler,(void *)pPhidget->id);
	
	// do phidget common open operations
	phidget_Open_Base((Phidget*)pPhidget);
	return TRUE;
}

//////////////////////////////////////////////////////////////////////////
// callback functions

/*++

Routine Description:

	This callback is called when the current draw of a motor of a Phidget
	Advanced Servo with the id pUserPtr	has changed.
	It signals the semaphore iSemaphoreCurrentChange.

Arguments:

	advServoHandle - handle of the Advanced Servo to signal the semaphore.

	pUserPtr - Callback function pointer.

	iIndex - index of the motor that generated the event

	dCurrent - the motor read current draw

Return Value:

	1 if the operation succeeded, 0 otherwise.

--*/
int CALLBACK phidgetAdvServo_currentChangeHandler(CPhidgetAdvancedServoHandle advServoHandle, void *pUserPtr, int iIndex, double dCurrent)
{
	PhidgetId id = (PhidgetId) pUserPtr;
	PhidgetAdvServo* pPhidget = (PhidgetAdvServo*) phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, 1);

	IF_DBG(SLog(SLOG_LEVEL_VERBOSE, "<AdvServo-CurrentChange> id = %i, semaphore = %i, index = %i, value = %f",
		pPhidget->id, pPhidget->iSemaphoreCurrentChange, iIndex, dCurrent));
	
	if (pPhidget->iSemaphoreCurrentChange > 0)
		interpreterProxy->signalSemaphoreWithIndex(pPhidget->iSemaphoreCurrentChange);
	else
		IF_DBG(SLog(SLOG_LEVEL_WARNING, "No semaphore set for <AdvServo-CurrentChange>, id = %i", pPhidget->id));
	return 0;
}

/*++

Routine Description:

	This callback is called when a motor of a Phidget Advanced Servo with 
	the id pUserPtr	has changed its velocity.
	It signals the semaphore iSemaphoreVelocityChange.

Arguments:

	advServoHandle - handle of the Advanced Servo to signal the semaphore.

	pUserPtr - Callback function pointer.

	iIndex - index of the motor that generated the event

	dVelocity - the motor read velocity

Return Value:

	1 if the operation succeeded, 0 otherwise.

--*/
int CALLBACK phidgetAdvServo_velocityChangeHandler(CPhidgetAdvancedServoHandle advServoHandle, void *pUserPtr, int iIndex, double dVelocity)
{
	PhidgetId id = (PhidgetId) pUserPtr;
	PhidgetAdvServo* pPhidget = (PhidgetAdvServo*) phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, 1);

	IF_DBG(SLog(SLOG_LEVEL_VERBOSE, "<AdvServo-VelocityChange> id = %i, semaphore = %i, index = %i, value = %f",
		pPhidget->id, pPhidget->iSemaphoreVelocityChange, iIndex, dVelocity));
	
	if (pPhidget->iSemaphoreVelocityChange > 0)
		interpreterProxy->signalSemaphoreWithIndex(pPhidget->iSemaphoreVelocityChange);
	else
		IF_DBG(SLog(SLOG_LEVEL_WARNING, "No semaphore set for <AdvServo-VelocityChange>, id = %i", pPhidget->id));
	return 0;
}
