/*++

Copyright (c) 2009
Lysann Kessler, Stephanie Platz, Thomas Klingbeil, Philipp Tessenow, Frank Schlegel;
Michael Haupt, Michael Perscheid, Robert Hirschfeld;
Software Architecture Group, Hasso-Plattner-Institut, University of Potsdam, Germany

This software is published under MIT license.
It uses phidget library by Phidgets Inc. (http://phidgets.com).
See ../../../../license.pdf for more information.

Module Name:

	PhidgetCompAxis.c

Abstract:

	This file contains methods for a Phidget Component Axis. 
	A component axis can belong to several phidgets e.g. the phidget Accelerometer.
	Only if one of these phidgets is attached the component is useable. Use 
	phidget_GetDeviceClass(pointerToPhidget) to call the specific phidget methods
	in phidget21 API.

--*/

#include "PhidgetCompAxis.h"

//////////////////////////////////////////////////////////////////////////
// methods

//
// Gets the number of acceleration axes supported by the owning phidget 
// of this component. Use id to identify the owning phidget.
//
int phidgetCompAxis_GetCount(PhidgetId id)
{
	int iCount = 0;
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, iCount);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
	case PHIDCLASS_ACCELEROMETER:
		CPhidgetAccelerometer_getAxisCount(((PhidgetAccel*) pPhidget)->handle, &iCount);
		break;

	default:
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompAxis_GetCount.", phidget_GetDeviceClass(pPhidget)));
	}

	return iCount;
}

//
// Gets the current acceleration of a component axis with the corresponding
// index. Use id to identify the owning phidget.
//
double phidgetCompAxis_GetAcceleration(PhidgetId id, int iIndex)
{
	double dAcceleration = 0.0;
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, dAcceleration);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
	case PHIDCLASS_ACCELEROMETER:
		CPhidgetAccelerometer_getAcceleration(((PhidgetAccel*) pPhidget)->handle, iIndex, &dAcceleration);
		break;

	default:
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompAxis_GetAcceleration.", phidget_GetDeviceClass(pPhidget)));
	}

	return dAcceleration;
}

//
// Gets the minimum accleration supported by a component axis with the
// corresponding index. Use id to identify the owning phidget.
//
double phidgetCompAxis_GetAccelerationMin(PhidgetId id, int iIndex)
{
	double dAcceleration = 0.0;
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, dAcceleration);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
	case PHIDCLASS_ACCELEROMETER:
		CPhidgetAccelerometer_getAccelerationMin(((PhidgetAccel*) pPhidget)->handle, iIndex, &dAcceleration);
		break;

	default:
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompAxis_GetAccelerationMin.", phidget_GetDeviceClass(pPhidget)));
	}

	return dAcceleration;
}

//
// Gets the maximum accleration supported by a component axis with the
// corresponding index. Use id to identify the owning phidget.
//
double phidgetCompAxis_GetAccelerationMax(PhidgetId id, int iIndex)
{
	double dAcceleration = 0.0;
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, dAcceleration);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
	case PHIDCLASS_ACCELEROMETER:
		CPhidgetAccelerometer_getAccelerationMax(((PhidgetAccel*) pPhidget)->handle, iIndex, &dAcceleration);
		break;

	default:
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompAxis_GetAccelerationMax.", phidget_GetDeviceClass(pPhidget)));
	}

	return dAcceleration;
}

/*++

Routine Description:

	This method sets a semaphore to be signaled when the phidget with the
	corresponding id has changed a components axis acceleration. 

Arguments:

	id - id of the Phidget to set the semaphore.

	iSemaphoreIndex - the semaphore the iSemaphoreAccelerationChange is set to.

Return Value:

	TRUE if the operation succeeded, FALSE otherwise.

--*/
BOOL phidgetCompAxis_SetSemaphoreIndex_AccelerationChange(PhidgetId id, int iSemaphoreIndex)
{
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, FALSE);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
	case PHIDCLASS_ACCELEROMETER:
		((PhidgetAccel*) pPhidget)->iSemaphoreAccelerationChange = iSemaphoreIndex;
		IF_DBG(SLog(SLOG_LEVEL_VERBOSE, "New semaphore for <Accel-AccelerationChange> is: %d, Phidget id = %i",
			((PhidgetRFID*) pPhidget)->iSemaphoreOutputChange, pPhidget->id));
		return TRUE;

	default:
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompAxis_SetSemaphoreIndex_AccelerationChange.", phidget_GetDeviceClass(pPhidget)));
		return FALSE;
	}
}
