/*++

Copyright (c) 2009
Lysann Kessler, Stephanie Platz, Thomas Klingbeil, Philipp Tessenow, Frank Schlegel;
Michael Haupt, Michael Perscheid, Robert Hirschfeld;
Software Architecture Group, Hasso-Plattner-Institut, University of Potsdam, Germany

This software is published under MIT license.
It uses phidget library by Phidgets Inc. (http://phidgets.com).
See ../../../../license.pdf for more information.

--*/

// INCLUDES

#include <simpleLog.h>
#include <time.h>
#include <stdarg.h>

// DEFINITIONS

#define SLOG_TIME_STRING_LENGTH	23	// "YYYY:mm:dd HH:MM:SS - \0"

// MACORS

#define SLOG_SAFE_FREE(x)				{ if(x != NULL) free(x); }
#define SLOG_SAFE_WRITE(string, file) 	\
	{if(string != NULL && file != NULL)	\
		fwrite(string, sizeof(char), strlen(string), file); }

#define SLOG_SAFE_LOCK()	\
	{ if(slog_bMutualExclusion && slog_pMutex != NULL) \
		xmutex_lock(slog_pMutex); }
#define SLOG_SAFE_UNLOCK()	\
	{ if(slog_bMutualExclusion && slog_pMutex != NULL) \
	xmutex_unlock(slog_pMutex); }

// GLOBAL VARIABLES

FILE* slog_pFile = NULL;						// pointer to the log file, or NULL if logging is not currently enabled
SLogLevel slog_level = SLOG_LEVEL_NOT_SET;		// minimal log level. messages below this level will not be printed.
SLOG_BOOL slog_bDebugVersion = SLOG_FALSE;		// Must be SLOG_TRUE to post debug messages.
SLOG_BOOL slog_bMutualExclusion = SLOG_TRUE;	// Must be SLOG_TRUE to use mutexes for log access
xmutex_t* slog_pMutex = NULL;					// pointer to mutex if latter value is SLOG_TRUE

// FORWARD DECLARATIONS

inline static const char* SLog_getLevelString(SLogLevel level);
inline char* SLog_getTimeString();

// FUCNTION DEFINITIONS

/*++
Routine Description:

	This function enables logging into the specified file and with given
	minimal log level. The log will use a Mutex to catch concurrent log
	accessing.
	
See also:

	SLog_enableLogging_ex()
	
Parameters:

	pcTargetFile - String describing the file name of the target file or NULL
		to use stdout. If the file already exists the new content will be
		appended to it. Else the file is created.
		
	minLevel - The minimal log level. All logs with smaller level will not be
		printed to the log file. Logs with level SLOG_LEVEL_DEBUG will not be
		logged. See SLog_enableLogging_ex() for more information.
		
Return Value:

	(See SLog_enableLogging_ex().)

Note:

	This function should not be called from more than 1 thread concurrently.
--*/
inline SLOG_BOOL SLog_enableLogging(const char* pcTargetFile, SLogLevel minLevel)
{
	return SLog_enableLogging_ex(pcTargetFile, "a", minLevel, SLOG_FALSE, SLOG_TRUE);
}

/*++
Routine Description:

	Extended function to enable logging. In addition to target file and minimal
	level, the write mode and a flag whether to print debug logs can be
	specified.

See also:

	SLog_enableLogging()
	
Parameters:

	pcTargetFile - String describing the file name of the target file or NULL
		to use stdout. If the file already exists the new content will be
		appended to it. Else the file is created.
	
	pcWriteMode - If pcTargetFile is a file name specify open flags here. See
		fopen() for more information.

	minLevel - The minimal log level. All logs with smaller level will not be
		printed to the log file.
	
	bDebugVersion - If SLOG_TRUE is specified, debug information will also be
		logged (logs with level SLOG_LEVEL_DEBUG).

	bMutualExclusion - If SLOG_TRUE is specified, the log will use mutual
		exclsions. Should be set, if more than one thread has access to the log
		at one time.
		
Return Value:

	SLOG_TRUE if logging is disabled and the file can be opened.
	SLOG_FALSE otherwise.

Note:

	This function should not be called from more than 1 thread concurrently.
--*/
inline SLOG_BOOL SLog_enableLogging_ex(const char* pcTargetFile, const char* pcWriteMode, SLogLevel minLevel, SLOG_BOOL bDebugVersion, SLOG_BOOL bMutualExclusion)
{
	if(slog_pFile != NULL) return SLOG_FALSE;

	// create the mutex first
	if(bMutualExclusion)
	{
		slog_pMutex = malloc(sizeof(xmutex_t));
		if(slog_pMutex != NULL)
			slog_bMutualExclusion = SLOG_TRUE;
	}

	SLOG_SAFE_LOCK();
	
	// open the log file: if NULL was given, open stdout
	if(pcTargetFile == NULL)
		slog_pFile = stdout;
	else
	{
		slog_pFile = fopen(pcTargetFile, pcWriteMode);
		if(slog_pFile == NULL)
		{
			SLOG_SAFE_UNLOCK();
			return SLOG_FALSE;
		}
	}
	
	// set the log level
	slog_level = minLevel;
	slog_bDebugVersion = bDebugVersion;

	SLOG_SAFE_UNLOCK();
	SLog(SLOG_LEVEL_INFO, "Logging enabled");
	return SLOG_TRUE;
}

/*++
Routine Description:

	This function disables logging that was previously enabled with
	SLog_enableLogging().
	Close the log file and reset global variables for logging.
		
Return Value:

	SLOG_TRUE if logging was enabled.
	SLOG_FALSE otherwise.
--*/
inline SLOG_BOOL SLog_disableLogging()
{
	if(slog_pFile == NULL) return SLOG_FALSE;
	
	SLog(SLOG_LEVEL_INFO, "Logging disabled");
	SLOG_SAFE_LOCK();

	// close the log file (if it is not a standard stream)
	if(slog_pFile != NULL && slog_pFile != stdout && slog_pFile != stderr)
		fclose(slog_pFile);
	slog_pFile = NULL;
	
	SLOG_SAFE_UNLOCK();
	return SLOG_TRUE;
}

/*++
Routine Description:

	Log an entry to the log file. The message contains the log level, log date
	and time and the logged message itself that can be passed in printf style.
	Logging must have been previously enabled using SLog_enableLogging().
	
Parameters:

	level - The level at which to post the message. If it is smaller than the
		minimal log level specified when enabling logging, it is not printed.
		Additionally, if the level is SLOG_LEVEL_DEBUG and debug logging was
		not enabled in SLog_enableLogging_ex(), the message is not printed.
	
	pcMessage - The log message to print. This can be a format string. Then the
		next arguments will be treated as additional arguments for sprintf.
		
Return Value:

	SLOG_TRUE if the entry was logged.
	SLOG_FALSE otherwise.
--*/
inline SLOG_BOOL SLog(SLogLevel level, const char* pcMessage, ...)
{
	char *pcTime = NULL;
	const char *pcLevel = NULL;
	va_list pArguments;

	SLOG_SAFE_LOCK();
	
	// SLog_enableLogging must have been called
	if(slog_pFile == NULL) return SLOG_FALSE;
	// the log level must be higher or equal to the current log level
	else if(level < slog_level) return SLOG_FALSE;
	// message must not be NULL
	else if(pcMessage == NULL) return SLOG_FALSE;
	// print debug messages only in a debug environment
	else if(level == SLOG_LEVEL_DEBUG && !slog_bDebugVersion) return SLOG_FALSE;
	
	// get the time string
	pcTime = SLog_getTimeString();
	// get the level as printable string
	pcLevel = SLog_getLevelString(level);
	
	// print log level and time to the file
	SLOG_SAFE_WRITE(pcTime, slog_pFile);
	SLOG_SAFE_WRITE(pcLevel, slog_pFile);
	
	// print formatted message to the file
	va_start(pArguments, pcMessage);
	vfprintf(slog_pFile, pcMessage, pArguments);
	va_end(pArguments);
	
	// print a newline and flush the file
	fprintf(slog_pFile, "\n");
	fflush(slog_pFile);

	// clean up
	SLOG_SAFE_UNLOCK();
	SLOG_SAFE_FREE(pcTime);
	
	return SLOG_TRUE;
}

// HELPER FUNCTIONS

/*++
Routine Description:

	Return a string describing a log level, followed by " - ".
	
Parameters:

	level - The level to describe.
		
Return Value:

	A constant C-String containing the descirption. It cannot be manipulated or freed.
--*/
inline static const char * SLog_getLevelString(SLogLevel level)
{
	switch(level)
	{
		case SLOG_LEVEL_VERBOSE: return "VERBOSE - ";
		case SLOG_LEVEL_INFO: return "INFO - ";
		case SLOG_LEVEL_WARNING: return "WARNING - ";
		case SLOG_LEVEL_ERROR: return "ERROR - ";
		case SLOG_LEVEL_CRITICAL: return "CRITICAL - ";
		case SLOG_LEVEL_DEBUG: return "DEBUG - ";
		default: return "UNKNOWN - ";
	}
}

/*++
Routine Description:

	Return a string describing the current date and time, followed by " - ".
		
Return Value:

	A constant C-String containing the time description. Must be freed after
	usage.
	Returns NULL if an error occurred.
--*/
inline char* SLog_getTimeString()
{
	time_t now;
	struct tm *pNow = NULL;
	char* pcTime = NULL;

	// get current time
	now = time(NULL);
	if(now != -1)
	{
		pNow = localtime(&now);
		if(pNow == NULL) return NULL;
		pcTime = malloc(sizeof(char) * SLOG_TIME_STRING_LENGTH);
		if(pcTime == NULL) return NULL;
		if(strftime(pcTime, SLOG_TIME_STRING_LENGTH, "%Y-%m-%d %H:%M:%S - ", pNow) == 0)
			return NULL;
	}
	
	return pcTime;
}
